/****************************************************************************
 *
 * Bewsserungssteuerung
 *
 * (C) Fingers elektrische Welt 2003
 *
 * Imagecraft-C
 * AT90S8515 @ 8 MHz
 * TAB-Space 4
 *
 ****************************************************************************/

#include <io8515v.h>
#include <macros.h>
#include <stdio.h>
#include "defines.h"

/*******************************************
 *
 * Variablen und Deklarationen
 *
 ******************************************/

// Prozessstruturdefinition
struct Prozessstruktur
 {
  char Art;                         // Tonne, Schlauch, etc
  char Status;                      // Beendet, laufend, Unterbrochen
  unsigned int Data [5];            // Datenfeld fr den jeweiligen Prozess
 };

// Prozessliste. Der gerade laufende Prozess steht immer an erster Stelle
struct Prozessstruktur Prozessliste [3];

struct Timer
 {
  char Timercount;
  long Ticker;
  char Blinkzaehler;
  char Timeoutzaehler_Pumpe;
 } Timerstruct;

// Diverses
char sprintfbuffer [40];					// Puffer zur sprintf-Ausgabe
char Blinker = FALSE;						// LED-Blinker
char Sekundenzaehler = 0;					// Ermittlung eines Sekundentaktes
int  Timeout_Beet = 120;					// Wird spter mit dem Poti eingestellt
char Pumpe_laeuft = FALSE;                  // Flag fr eine laufende Pumpe
char Fehlerflag = FALSE;          // In ISR aufgetretener Fehler

/*******************************************
 *
 * Initialisierungsroutinen
 *
 ******************************************/
#include "init.c"

/*******************************************
 *
 * Ermittelt die Position eines Prozesses
 *
 ******************************************/
char Finde_Prozess (char Art)
 {
  if (Prozessliste [0].Art == Art) return (0);
  if (Prozessliste [1].Art == Art) return (1);
  if (Prozessliste [2].Art == Art) return (2);
  return (0);
 }


/*******************************************
 *
 * Zeigt die komplette Prozessliste
 *
 ******************************************/
/*
void zeige_prozesse (char Art)
 {
  char i = 0;

  if (Art) sprintf (sprintfbuffer, "Taste:\n\r"); else
           sprintf (sprintfbuffer, "Scheduler:\n\r");

  puts (sprintfbuffer);

  for (i=0; i < 3; i++)
   {
    sprintf (sprintfbuffer, "%d %d\r",Prozessliste [i].Art, Prozessliste [i].Status);
    puts (sprintfbuffer);
   }

 }
*/

/*******************************************
 *
 * Schaltet Relais ein / aus
 *
 ******************************************/
void Relais (char Nummer, char Status)
 {

  switch (Nummer)
   {
    case _Pumpe : {
                    if (Status)
                     {
                      PORTD &= ~Relais_Pumpe; // Ein
                      Pumpe_laeuft = TRUE;
                     } else
                     {

                      PORTD |= Relais_Pumpe; // Aus
                      Pumpe_laeuft = FALSE;
                     }

                    break;
                   }

    case _230V : {
                    if (Status) PORTD |= Relais_230V; else PORTD &= ~Relais_230V;
                     break;
                   }
    case _Tonne : {
                    if (Status) PORTD |= Relais_Tonne; else PORTD &= ~Relais_Tonne;
                     break;
                   }
    case _Schlauch : {
                    if (Status) PORTD |= Relais_Schlauch; else PORTD &= ~Relais_Schlauch;
                     break;
                   }
    case _Beet : {
                    if (Status) PORTD |= Relais_Beet; else PORTD &= ~Relais_Beet;
                     break;
                   }
    case _Vert1A : {
                    if (Status) PORTB |= Relais_Vert1B; else PORTB &= ~Relais_Vert1B;
                     break;
                   }
    case _Vert1B : {
                    if (Status) PORTD |= Relais_Vert1A; else PORTD &= ~Relais_Vert1A;
                     break;
                   }
    case _Vert2A : {
                           if (Status) PORTB |= Relais_Vert2B; else PORTB &= ~Relais_Vert2B;
                            break;
                          }
     case _Vert2B : {
                    if (Status) PORTB |= Relais_Vert2A; else PORTB &= ~Relais_Vert2A;
                     break;
                   }
    case _Vert3A : {
                    if (Status) PORTB |= Relais_Vert3A; else PORTB &= ~Relais_Vert3A;
                     break;
                   }
    case _Vert3B : {
                           if (Status) PORTB |= Relais_Vert3B; else PORTB &= ~Relais_Vert3B;
                            break;
                          }

   } // Von Switch

 }


/*******************************************
 *
 * Wird 10* pro Sekunde aufgerufen
 *
 ******************************************/
void Zehnteltimer (void)
 {
  char Prozessnummer;

  //------------
  // LED-Blinker
  //------------
  Timerstruct.Blinkzaehler++;
  if (Timerstruct.Blinkzaehler == Blinkertimer)
   {
    Timerstruct.Blinkzaehler = 0;
    if (Blinker) Blinker = FALSE; else Blinker = TRUE;
   }

  //-----------------
  // Pegelstands-LED
  //-----------------
  if (!(PINA & Sensor_Pegel))
   {
    PORTC |= LED_Tonne_gruen;				// Tonne voll
    PORTC &= ~LED_Tonne_rot;
   } else
   {
    PORTC &= ~LED_Tonne_gruen;      // Tonne leer
    PORTC |= LED_Tonne_rot;
   }

  //--------------------------------
  // Prozess-LEDs und Pumpe/Ventile
  //--------------------------------

  // Wenn etwas luft, dann Pumpe und Ventilversorgung einschalten, falls Automatikbetrieb
 if (!(PINA & Schalter_Auto))
  {

  if (Prozessliste [0].Status == Laufend)
   {
    Relais (_Pumpe, EIN);
    Relais (_230V, EIN);

    switch (Prozessliste [0].Art)
     {

      case (Regentonne) : {
                           Relais (_Tonne, EIN);
                           Relais (_Schlauch, AUS);
                           Relais (_Beet, AUS);

                           Relais (_Vert1A, AUS);
                           Relais (_Vert1B, AUS);
                           Relais (_Vert2A, AUS);
                           Relais (_Vert2B, AUS);
                           Relais (_Vert3A, AUS);
                           Relais (_Vert3B, AUS);
       					   break;
      					  }

      case (Schlauch) :   {
                           Relais (_Tonne, AUS);
                           Relais (_Schlauch, EIN);
                           Relais (_Beet, AUS);

                           Relais (_Vert1A, AUS);
                           Relais (_Vert1B, AUS);
                           Relais (_Vert2A, AUS);
                           Relais (_Vert2B, AUS);
                           Relais (_Vert3A, AUS);
                           Relais (_Vert3B, AUS);
       					   break;
      					  }

      case (Beet) :  		  {
                           Relais (_Tonne, AUS);
                           Relais (_Schlauch, AUS);
                           Relais (_Beet, EIN);

                           Relais (_Vert1A, AUS);
                           Relais (_Vert1B, AUS);
                           Relais (_Vert2A, AUS);
                           Relais (_Vert2B, AUS);
                           Relais (_Vert3A, AUS);
                           Relais (_Vert3B, AUS);

                           // Steuerung der Verteilerinseln je nach Beetnummer
                           switch (Prozessliste [0].Data [Beetnummer])
                            {
                             case (0) : { // Beet 1
                             			 Relais (_Vert1A, EIN);
                              			 break;
                             			}

                             case (1) : { // Beet 2
                                         Relais (_Vert1B, EIN);
                                         break;
                             			}

                             case (2) : { // Beet 3
                                         Relais (_Vert2A, EIN);
                              			 break;
                             			}

                             case (3) : { // Beet 4
                                         Relais (_Vert2B, EIN);
                              			 break;
                             			}


                            } // Von Switch

       					 break;
      					}


     } // Von Switch

   } else

   { // Alles abschalten
     Relais (_Pumpe, AUS);
     Relais (_230V, AUS);

     Relais (_Tonne, AUS);
     Relais (_Schlauch, AUS);
     Relais (_Beet, AUS);

     Relais (_Vert1A, AUS);
     Relais (_Vert1B, AUS);
     Relais (_Vert2A, AUS);
     Relais (_Vert2B, AUS);
     Relais (_Vert3A, AUS);
     Relais (_Vert3B, AUS);
   }

  //-----------
  // Regentonne
  //-----------
  Prozessnummer = Finde_Prozess (Regentonne);
  if (Prozessliste [Prozessnummer].Status == Beendet) 					 PORTC |= LED_Tonne;  // LED aus
  if (Prozessliste [Prozessnummer].Status == Laufend) 					 PORTC &= ~LED_Tonne; // LED ein
  if ( (Prozessliste [Prozessnummer].Status == Unterbrochen) &&
       (Blinker) )	 																						 PORTC &= ~LED_Tonne; // LED blinken
  if ( (Prozessliste [Prozessnummer].Status == Unterbrochen) &&
       !(Blinker) )	 																						 PORTC |= LED_Tonne;  // LED blinken

  //----------
  // Schlauch
  //----------
  Prozessnummer = Finde_Prozess (Schlauch);
  if (Prozessliste [Prozessnummer].Status == Beendet) 					 PORTC |= LED_Schlauch;  // LED aus
  if (Prozessliste [Prozessnummer].Status == Laufend) 					 PORTC &= ~LED_Schlauch; // LED ein
  if ( (Prozessliste [Prozessnummer].Status == Unterbrochen) &&
       (Blinker) )	 																						 PORTC &= ~LED_Schlauch; // LED blinken
  if ( (Prozessliste [Prozessnummer].Status == Unterbrochen) &&
       !(Blinker) )	 																						 PORTC |= LED_Schlauch;  // LED blinken

  //-----
  // Beet
  //-----
  Prozessnummer = Finde_Prozess (Beet);
  if (Prozessliste [Prozessnummer].Status == Beendet)
   {
    PORTC |= LED_Beet1;  // LEDs aus
    PORTC |= LED_Beet2;  // LEDs aus
    PORTC |= LED_Beet3;  // LEDs aus
   }

  if (Prozessliste [Prozessnummer].Status == Laufend)
   switch (Prozessliste [Prozessnummer].Data [Beetnummer])
    {

     case 0 : {
                PORTC &= ~LED_Beet1;
    						PORTC |= LED_Beet2;
    						PORTC |= LED_Beet3;
                break;
     				  }

     case 1 : {
                PORTC |= LED_Beet1;
    						PORTC &= ~LED_Beet2;
    						PORTC |= LED_Beet3;
                break;
     				  }

     case 2 : {
                PORTC |= LED_Beet1;
    			PORTC |= LED_Beet2;
    			PORTC &= ~LED_Beet3;
                break;
     		  }

     case 3 : {
                PORTC &= ~LED_Beet1;
    			PORTC |= LED_Beet2;
    			PORTC &= ~LED_Beet3;
                break;
     		  }


    } // Von Switch

  if (Prozessliste [Prozessnummer].Status == Unterbrochen)
   switch (Prozessliste [Prozessnummer].Data [Beetnummer])
    {

     case 0 : {
                if (Blinker) PORTC &= ~LED_Beet1; else PORTC |= LED_Beet1;
    						PORTC |= LED_Beet2;
    						PORTC |= LED_Beet3;
                break;
     				  }

     case 1 : {
                PORTC |= LED_Beet1;
    						if (Blinker) PORTC &= ~LED_Beet2; else PORTC |= LED_Beet2;
    						PORTC |= LED_Beet3;
                break;
     				  }

     case 2 : {
                PORTC |= LED_Beet1;
    						PORTC |= LED_Beet2;
    						if (Blinker) PORTC &= ~LED_Beet3; else PORTC |= LED_Beet3;
                break;
     				  }

    } // Von Switch

  } // Von if (!(PINA & Schalter_Auto))

 }

/******************************************
 *
 * Fehlerbehandlung
 *
 ******************************************/
void fehler (void)
 {
   Relais (_Pumpe, AUS); // Pumpe aus
   Relais (_230V, AUS);  // Alle Ventile aus

   sprintf (sprintfbuffer, "--- FEHLER ---");
   puts (sprintfbuffer);

   while (1) 							 // LED Fehler blinkt, Ende mit allem
    {
     if (Blinker) PORTC |= LED_Fehler; else PORTC &= ~LED_Fehler;
     WDR ();
    }
 }

/*******************************************
 *
 * Prozessberwachung
 *
 ******************************************/
void Prozesscheck (void)
 {

  // Luft der Prozess Tonne ?
  if ( (Prozessliste [0].Art == Regentonne) && (Prozessliste [0].Status == Laufend) )
   {
    Prozessliste [0].Data [Prozesstimeout]++;
    return;
   }

  // Er luft nicht, also Timeout zurcksetzen
  // Prozessliste [0].Data [Prozesstimeout] = 0;

  // Luft der Prozess Beet ?
  if ( (Prozessliste [0].Art == Beet) && (Prozessliste [0].Status == Laufend) )
   {
    Prozessliste [0].Data [Prozesstimeout]++;
    return;
   }

  // Luft der Prozess Schlauch ?
  if ( (Prozessliste [0].Art == Schlauch) && (Prozessliste [0].Status == Laufend) )
   {
    // Druck und Durchfluss vorhanden ?
    if ( (!(PINA & Sensor_Druck)) && (PINA & Sensor_Durchfluss) ) Prozessliste [0].Data [Prozesstimeout]++; else
     if (PINA & Sensor_Druck) Prozessliste [0].Data [Prozesstimeout] = 0;
   }

 }


/*******************************************
 *
 * Trockenlaufberwachung
 *
 ******************************************/
void Trockenlauf (void)
 {
  // Wenn die Pumpe nicht luft kann nix passieren...
  if (!(Pumpe_laeuft))
   {
    Timerstruct.Timeoutzaehler_Pumpe = 0;
    return;
   }

  // Die Pumpe luft also....
  if ( (PINA & Sensor_Durchfluss) && (PINA & Sensor_Druck) )     // Kein Durchfluss und kein Druck ?
   {
     Timerstruct.Timeoutzaehler_Pumpe++;
     if (Timerstruct.Timeoutzaehler_Pumpe > Timeout_Pumpe) Fehlerflag = TRUE;  // Wenn 2 Minuten kein Durchfluss und kein Druck
   } else Timerstruct.Timeoutzaehler_Pumpe = 0;

 }

/*******************************************
 *
 * Wird jede Sekunde aufgerufen
 *
 ******************************************/
void Sekundentimer (void)
 {
  Prozesscheck ();										// Prozessberwachung
  Trockenlauf ();											// Trockenlaufberwachung
 }

/*******************************************
 *
 * Timer-0 ISR, Ansprung alle 2ms
 *
 ******************************************/
#pragma interrupt_handler timer0_ovf_isr:8
void timer0_ovf_isr(void)
 {
  TCNT0 = 0x06; 											// Reload Timer

  // Ablauf alle 100ms
  Timerstruct.Timercount++;
  if (Timerstruct.Timercount == 50)
   {
    Timerstruct.Timercount = 0;
    Timerstruct.Ticker++;

    Zehnteltimer ();

    //-----------------------
    // Sekundenweise Ablufe
    //-----------------------
    Sekundenzaehler++;
    if (Sekundenzaehler == 10)
     {
      Sekundenzaehler = 0;
      Sekundentimer ();
     }

    // Weitere Applikationen



   } // Von "Ablauf alle 100ms"

 }

/*******************************************
 *
 * Wartet time*100ms
 *
 ******************************************/
void Delay (char time)
 {
  long temp;

  temp = Timerstruct.Ticker;
  while ( (temp+time) > Timerstruct.Ticker) WDR ();
 }

/*******************************************
 *
 * Vertauscht 2 Prozesse miteinander
 *
 ******************************************/
void Prozesswap (char Prozess1, char Prozess2)
 {
  struct Prozessstruktur Swappuffer;
  Swappuffer = Prozessliste [Prozess1];
  Prozessliste [Prozess1] = Prozessliste [Prozess2];
  Prozessliste [Prozess2] = Swappuffer;

  /*
  sprintf (sprintfbuffer, "Swap %d %d\n\r",Prozess1,Prozess2);
  puts (sprintfbuffer);
  */

 }

/*******************************************
 *
 * Startet einen Prozess
 *
 ******************************************/
void starte_prozess (char Art)
 {

  sprintf (sprintfbuffer, "Start %d\n\r", Art);
  puts (sprintfbuffer);

  if (Prozessliste [0].Art != Art)							// Ist der Prozess bereits auf Platz 1 ?
   {
    if (Prozessliste [0].Status != Beendet)     // Prozess auf Platz 1 unterbrechen, sofern er luft
     Prozessliste [0].Status = Unterbrochen;

    // Position ermitteln und mit erstem Platz tauschen
    if (Prozessliste [1].Art == Art) Prozesswap (0,1); else
                                     Prozesswap (0,2);
   }

  // Prozessspezifische Daten erzeugen
  if (Art == Regentonne)
   {
    Prozessliste [0].Data [Prozesstimeout] = 0;
   }

  if (Art == Schlauch)
   {
    Prozessliste [0].Data [Prozesstimeout] = 0;
   }

  if (Art == Beet)
   {
    Prozessliste [0].Data [Prozesstimeout] = 0;
    Prozessliste [0].Data [Beetnummer] = 0;
   }

  // Prozess starten
  Prozessliste [0].Status = Laufend;
 }


/**********************************************************
 *
 * Startet einen prozess und unterbricht ihn gleichzeitig
 *
 *********************************************************/
void starte_Prozess_unterbrochen (char Art)
 {
  char Prozessnummer;

  sprintf (sprintfbuffer, "Start_U %d\n\r", Art);
  puts (sprintfbuffer);

  Prozessnummer = Finde_Prozess (Art);

  // Prozessspezifische Daten erzeugen
  if (Art == Regentonne)
   {
    Prozessliste [0].Data [Prozesstimeout] = 0;
   }

  if (Art == Schlauch)
   {
    Prozessliste [0].Data [Prozesstimeout] = 0;
   }

  if (Art == Beet)
   {
    Prozessliste [0].Data [Prozesstimeout] = 0;
    Prozessliste [0].Data [Beetnummer] = 0;
   }

  // Prozess unterbrechen
  Prozessliste [Prozessnummer].Status = Unterbrochen;

 }

/*******************************************
 *
 * Beendet einen Prozess
 *
 ******************************************/
void beende_Prozess (char Art)
 {
  char Prozessnummer;

  /*
  sprintf (sprintfbuffer, "------");
  puts (sprintfbuffer);

  zeige_prozesse (TRUE);

  sprintf (sprintfbuffer, "Ende %d\n\r", Art);
  puts (sprintfbuffer);
  */

  // Prozess beenden
  Prozessnummer = Finde_Prozess (Art);
  Prozessliste [Prozessnummer].Status = Beendet;

  // zeige_prozesse (TRUE);

  // Wenn der Prozess auf Platz 1 lag, nachschauen, ob noch andere da sind......
  if (Prozessnummer == 0)
   {

    // Sind noch andere Prozesse dran ? Dann auf Platz 1 legen und aktiveren
    if (Prozessliste [0].Status == Unterbrochen)
     {
      Prozessliste [0].Status = Laufend;						// Prozess starten
     } else

    if (Prozessliste [1].Status == Unterbrochen)
     {
      Prozesswap (0,1);
      Prozessliste [0].Status = Laufend;						// Prozess starten
     } else

    if (Prozessliste [2].Status == Unterbrochen)
     {
      Prozesswap (0,2);
      Prozessliste [0].Status = Laufend;						// Prozess starten
     }

   }

  /*
  zeige_prozesse (TRUE);

  sprintf (sprintfbuffer, "------");
  puts (sprintfbuffer);
  */

 }

/*******************************************
 *
 * Reaktiviert einen unterbrochenen Prozess
 *
 ******************************************/
 void reaktiviere_Prozess (char Art)
  {
   char Prozessnummer;

   Prozessnummer = Finde_Prozess (Art);
   if (Prozessnummer != 0)
    {
     if (Prozessliste [0].Status == Laufend) Prozessliste [0].Status = Unterbrochen;
     Prozesswap (0,Prozessnummer);
    }

   Prozessliste [0].Status = Laufend;							// Prozess starten
  }


/********************************************************
 *
 * Erlaubt die Steuerung der Anlage von Hand
 *
 ********************************************************/
void Handschaltung (void)
 {
  char Hand_Tonne = FALSE;
  char Hand_Schlauch = FALSE;
  char Hand_Beet = 0;

  // Pumpe und Ventilversorgung ein
  Relais (_Pumpe, EIN);
  Relais (_230V, EIN);

  // Alle Ventile zu
  Relais (_Tonne, AUS);
  Relais (_Schlauch, AUS);
  Relais (_Beet, AUS);

  Relais (_Vert1A, AUS);
  Relais (_Vert1B, AUS);
  Relais (_Vert2A, AUS);
  Relais (_Vert2B, AUS);
  Relais (_Vert3A, AUS);
  Relais (_Vert3B, AUS);

  // Alle LEDs aus
  PORTC |= LED_Tonne;
  PORTC |= LED_Schlauch;
  PORTC |= LED_Beet1;
  PORTC |= LED_Beet2;
  PORTC |= LED_Beet3;

  sprintf (sprintfbuffer, "Hand\n\r");
  puts (sprintfbuffer);

  // Solange abarbeiten, bis wieder auf Automatik umgeschaltet wurde
  while (PINA & Schalter_Auto)
   {

    WDR ();

    //-----------------
    // Taster bedienen
    //-----------------

    // Taster Tonne
    if (PINA & Taster_Tonne)
     {
      Delay (2);															// Warte 100 ms
      while (PINA & Taster_Tonne) WDR ();  		// Warte, bis losgelassen
      if (Hand_Tonne)
       { // Ventil schliessen, LED aus
         Hand_Tonne = FALSE;
         PORTC |= LED_Tonne;
         Relais (_Tonne, AUS);
         sprintf (sprintfbuffer, "Tonne aus\n\r");
         puts (sprintfbuffer);
       } else
       { // Ventil ffnen, LED ein
         Hand_Tonne = TRUE;
         PORTC &= ~LED_Tonne;
         Relais (_Tonne, EIN);
         sprintf (sprintfbuffer, "Tonne ein\n\r");
         puts (sprintfbuffer);
       }
     } // Von Taster Tonne

    // Taster Schlauch
    if (PINA & Taster_Schlauch)
     {
      Delay (2);															// Warte 100 ms
      while (PINA & Taster_Schlauch) WDR ();  // Warte, bis losgelassen
      if (Hand_Schlauch)
       { // Ventil schliessen, LED aus
         Hand_Schlauch = FALSE;
         PORTC |= LED_Schlauch;
         Relais (_Schlauch, AUS);
         sprintf (sprintfbuffer, "Schlauch aus\n\r");
         puts (sprintfbuffer);
       } else
       { // Ventil ffnen, LED ein
         Hand_Schlauch = TRUE;
         PORTC &= ~LED_Schlauch;
         Relais (_Schlauch, EIN);
         sprintf (sprintfbuffer, "Schlauch ein\n\r");
         puts (sprintfbuffer);
       }
     } // Von Taster Schlauch

    // Taster Beet
    if (PINA & Taster_Beet)
     {
      Delay (2);															// Warte 100 ms
      while (PINA & Taster_Beet) WDR ();  	  // Warte, bis losgelassen

      PORTC |= LED_Beet1;
      PORTC |= LED_Beet2;
      PORTC |= LED_Beet3;

      // Alle Ventile zu......
      Relais (_Vert1A, AUS);
      Relais (_Vert1B, AUS);
      Relais (_Vert2A, AUS);
      Relais (_Vert2B, AUS);
      Relais (_Vert3A, AUS);
      Relais (_Vert3B, AUS);

      if (Hand_Beet > Beetanzahl)
       { // Ventile schliessen, LED aus
         Hand_Beet = 0;
         Relais (_Beet, AUS);
         sprintf (sprintfbuffer, "Beet aus\n\r");
         puts (sprintfbuffer);
         // penis
       } else
       { // Ventile ffnen, LED ein
         Relais (_Beet, EIN);

         // Beet 1 angewhlt
         if (Hand_Beet == 0)
          {
           PORTC &= ~LED_Beet1;
           Relais (_Vert1A, EIN);
           sprintf (sprintfbuffer, "Beet 1 ein\n\r");
           puts (sprintfbuffer);
          } else

         // Beet 2 angewhlt
         if (Hand_Beet == 1)
          {
           PORTC &= ~LED_Beet2;
           Relais (_Vert1B, EIN);
           sprintf (sprintfbuffer, "Beet 2 ein\n\r");
           puts (sprintfbuffer);
          } else

         // Beet 3 angewhlt
         if (Hand_Beet == 2)
          {
           PORTC &= ~LED_Beet3;
           Relais (_Vert2A, EIN);
           sprintf (sprintfbuffer, "Beet 3 ein\n\r");
           puts (sprintfbuffer);
          }

         // Beet 4 angewhlt
         if (Hand_Beet == 3)
          {
           PORTC &= ~LED_Beet1;		  
           PORTC &= ~LED_Beet3;
           Relais (_Vert2B, EIN);
           sprintf (sprintfbuffer, "Beet 4 ein\n\r");
           puts (sprintfbuffer);
          }

          Hand_Beet++;

       }
//penis
     } // Von Taster Beet

   }

  sprintf (sprintfbuffer, "Hand aus\n\r");
  puts (sprintfbuffer);

 }

/********************************************************
 *
 * Ermittelt die Stellung des Potis
 *
 ********************************************************/
void get_poti (void)
 {
   int  Potiwert = 0;
   int Delaytime = 0;
   int Temp = 0;

   DDRA |= Sensor_Zeit;					 // Pin auf Ausgang schalten
   PORTA |= Sensor_Zeit;		 		 // Pin auf 1 setzen
   Delay (1);										 // 100ms warten
   DDRA &= ~Sensor_Zeit;				 // Pin auf Eigang schalten
   PORTA &=~Sensor_Zeit;				 // Pin auf Tristate schalten

   // Warten, bis der Pin auf Low geht
   while (PINA & Sensor_Zeit)
    {
     for (Delaytime = 0; Delaytime < 100; Delaytime++) Temp = Delaytime / 2;
     Potiwert++;
    }
   // Zeit berechnen (60s bis 900s, entsprechend 1-15min)
   Timeout_Beet = ((Potiwert * 70) / 3) + 60;
 }

/********************************************************
 *
 * Fragt die Tasten ab und steuert demnach die Prozesse
 *
 ********************************************************/
void Tastaturabfrage (void)
 {
   char Prozessnummer;

  //-------------------
  // Taster Regentonne
  //-------------------
  if (PINA & Taster_Tonne)
   {
    Delay (2);															// Warte 100 ms
    while (PINA & Taster_Tonne) WDR ();	  	// Warte, bis losgelassen
    Prozessnummer = Finde_Prozess (Regentonne);

    sprintf (sprintfbuffer, "T_Tonne\n\r");
    puts (sprintfbuffer);

    // Ist der Prozess unterbrochen ? Dann reaktivieren
    if (Prozessliste [Prozessnummer].Status == Unterbrochen) reaktiviere_Prozess (Regentonne);

    // zeige_prozesse (TRUE);

   } // Von Taster Regentonne

  //-----------------
  // Taster Schlauch
  //-----------------
  if (PINA & Taster_Schlauch)
   {
    Delay (2);															// Warte 100 ms
    while (PINA & Taster_Schlauch) WDR ();	// Warte, bis losgelassen
    Prozessnummer = Finde_Prozess (Schlauch);

    sprintf (sprintfbuffer, "T_Schlauch\n\r");
    puts (sprintfbuffer);

    // Luft der Prozess ? Dann beenden
    if ( (Prozessliste [0].Status == Laufend) &&
         (Prozessliste [0].Art == Schlauch) ) beende_Prozess (Schlauch); else
    // Ist der Prozess beendet ? Dann starten
    if (Prozessliste [Prozessnummer].Status == Beendet) starte_prozess (Schlauch); else
    // Ist der Prozess unterbrochen ? Dann reaktivieren
    if (Prozessliste [Prozessnummer].Status == Unterbrochen) reaktiviere_Prozess (Schlauch);

    // zeige_prozesse (TRUE);

   } // Von Taster Schlauch

  //-------------
  // Taster Beet
  //-------------
  if (PINA & Taster_Beet)
   {
    Delay (1);															// Warte 100 ms
    while (PINA & Taster_Beet) WDR ();		  // Warte, bis losgelassen
    Prozessnummer = Finde_Prozess (Beet);

    sprintf (sprintfbuffer, "T_Beet\n\r");
    puts (sprintfbuffer);

    // Luft der Prozess ? Dann weiterschalten bzw. beenden
    if ( (Prozessliste [0].Status == Laufend) &&
         (Prozessliste [0].Art == Beet) )
     {
      Prozessliste [0].Data [Beetnummer]++;
      if (Prozessliste [0].Data [Beetnummer] > Beetanzahl) beende_Prozess (Beet);
     } else
    // Ist der Prozess beendet ? Dann starten
    if (Prozessliste [Prozessnummer].Status == Beendet) starte_prozess (Beet); else
    // Ist der Prozess unterbrochen ? Dann reaktivieren
    if (Prozessliste [Prozessnummer].Status == Unterbrochen) reaktiviere_Prozess (Beet);

    // zeige_prozesse (TRUE);

   } // Von Taster Beet

  //---------------------
  // Potiwert reinholen
  //---------------------
  get_poti ();

  //-------------------------
  // Schalter Hand/Automatik
  //-------------------------
  if (PINA & Schalter_Auto) Handschaltung ();

 }

/**********************************
 *
 * Wickelt die Prozesssteuerung ab
 *
 *********************************/
void Scheduler (void)
 {
  char Prozessnummer;

  // Prozess Schlauch aktiv ?
  if ( (Prozessliste [0].Art == Schlauch) && (Prozessliste [0].Status == Laufend) &&
       (Prozessliste [0].Data [Prozesstimeout] > Timeout_Schlauch) ) beende_Prozess (Schlauch);

  // Prozess Beet aktiv ?
  if ( (Prozessliste [0].Art == Beet) && (Prozessliste [0].Status == Laufend) &&
       (Prozessliste [0].Data [Prozesstimeout] > Timeout_Beet) )
   {
     Prozessliste [0].Data [Prozesstimeout] = 0;
     Prozessliste [0].Data [Beetnummer]++;
     if (Prozessliste [0].Data [Beetnummer] > Beetanzahl) beende_Prozess (Beet);
   }

  // Niedriger Pegel in der Tonne ?
  if (!(PINA & Sensor_Pegel))
   { // Tonne ist voll
     Prozessnummer = Finde_Prozess (Regentonne);
     if (Prozessliste [Prozessnummer].Status != Beendet) beende_Prozess (Regentonne);
   }  else

   { // Tonne ist leer

   //--------------------------------------------------------------------------------------------

    // Prozess Tonne aktiv ?
    if ( (Prozessliste [0].Art == Regentonne) && (Prozessliste [0].Status == Laufend) &&
          (Prozessliste [0].Data [Prozesstimeout] > Timeout_Tonne) ) Fehlerflag = TRUE; else
     { // Prozess Tonne ist nicht aktiv

      // Ist der Prozess unterbrochen ?
      Prozessnummer = Finde_Prozess (Regentonne);

      // Timeout rcksetzen
      if (Prozessliste [Prozessnummer].Status == Unterbrochen)
       Prozessliste [Prozessnummer].Data [Prozesstimeout] = 0;

      // Wenn nicht und nichts anderes luft, dann starten
      if ( (Prozessliste [Prozessnummer].Status != Unterbrochen) &&
         (Prozessliste [0].Status == Beendet) ) starte_prozess (Regentonne); else

      // Wenn nicht und etwas luft, dann unterbochen starten
      if ( (Prozessliste [Prozessnummer].Status != Unterbrochen) &&
         (Prozessliste [0].Status == Laufend) &&
         (Prozessnummer != 0) ) starte_Prozess_unterbrochen (Regentonne);

     }

   //--------------------------------------------------------------------------------------------

   } // Von Tonne ist leer

 }

/*****************************
 *
 * Hauptprogramm
 *
 *****************************/
void main (void)
 {
  init_devices ();
  Init_Prozesse ();

  sprintf (sprintfbuffer, "Init\n\r");
  puts (sprintfbuffer);

  while (1)
   {
    Tastaturabfrage ();
    Scheduler ();
    if (Fehlerflag) fehler ();
    WDR ();
   }

 }
